package com.tzld.longarticle.recommend.server.service;

import com.tzld.longarticle.recommend.server.model.ArticleSortResponseData;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.model.RecommendRequest;
import com.tzld.longarticle.recommend.server.model.RecommendResponse;
import com.tzld.longarticle.recommend.server.service.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.rank.RankService;
import com.tzld.longarticle.recommend.server.service.recall.RecallParam;
import com.tzld.longarticle.recommend.server.service.recall.RecallResult;
import com.tzld.longarticle.recommend.server.service.recall.RecallService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecommendService {

    @Autowired
    private RecallService recallService;
    @Autowired
    private RankService rankService;


    public RecommendResponse recommend(RecommendRequest request) {

        RecommendParam param = genRecommendParam(request);

        RecallResult recallResult = recallService.recall(convertToRecallParam(param));
        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));
        // MergeResult mergeResult = mergeService.merge(convertToMergeParam(param, rankResult));

        List<Content> contentList = rankResult.getContents();
        List<Content> filterContentList = new ArrayList<>();
        recallResult.getData().forEach(item -> filterContentList.addAll(item.getFilterContents()));
        ArticleSortResponseData data = new ArticleSortResponseData();
        data.setRank_list(contentList.subList(0, Math.min(rankResult.getContents().size(), param.getPublishNum())));
        data.setFilter_list(filterContentList);

        RecommendResponse response = new RecommendResponse();
        response.setCode(0);
        response.setMsg("success");
        response.setData(data);
        return response;
    }

    public RecommendParam genRecommendParam(RecommendRequest request) {
        RecommendParam param = new RecommendParam();
        BeanUtils.copyProperties(request, param);
        return param;
    }

    public RecallParam convertToRecallParam(RecommendParam param) {
        RecallParam recallParam = new RecallParam();
        BeanUtils.copyProperties(param, recallParam);
        return recallParam;
    }


    public RankParam convertToRankParam(RecommendParam param, RecallResult recallResult) {
        RankParam rankParam = new RankParam();
        List<Content> contentList = new ArrayList<>();
        if (Objects.nonNull(recallResult) && CollectionUtils.isNotEmpty(recallResult.getData())) {
            recallResult.getData().forEach(item -> contentList.addAll(item.getContents()));
        }
        rankParam.setContents(contentList);
        return rankParam;
    }

}
