package com.tzld.longarticle.recommend.server.service.rank;


import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.score.ScoreService;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ContentPoolStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.SimilarityStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ViewCountStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService {

    @Autowired
    private ScoreService scoreService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {

        ScoreResult scoreResult = scoreService.score(convertToScoreParam(param));
        Map<String, Map<Class<? extends ScoreStrategy>, Double>> scoreMap = scoreResult.getScoreMap();

        List<RankItem> items = CommonCollectionUtils.toList(param.getContents(), c -> {
            Map<Class<? extends ScoreStrategy>, Double> map = scoreMap.get(c.getId());
            double score = 1 * map.getOrDefault(SimilarityStrategy.class, 0.0)
                    + 10 * map.getOrDefault(ViewCountStrategy.class, 0.0)
                    + 100 * map.getOrDefault(ContentPoolStrategy.class, 0.0);

            RankItem item = new RankItem();
            item.setContent(c);
            item.setScore(score);
            return item;

        });
        Collections.sort(items, Comparator.comparingDouble(o -> -o.getScore()));

        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);
        // 1 相似去重
        contents = deduplication(contents);
        // 2 文章按照内容池分组
        Map<String, List<Content>> contentMap = new HashMap<>();
        for (Content c : contents) {
            List<Content> data = contentMap.computeIfAbsent(c.getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }
        // 3 按位置选文章
        List<Content> result = new ArrayList<>();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());

        // 头
        List<Content> pool = contentMap.get(contentPools[0]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.add(pool.get(RandomUtils.nextInt(0, Math.min(pool.size(), 5))));
        }

        // 次
        pool = contentMap.get(contentPools[1]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.add(pool.get(0));
            if (result.size() == 1 && pool.size() > 1) {
                result.add(pool.get(1));
            }
        }

        // 3-8
        pool = contentMap.get(contentPools[2]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.addAll(pool.subList(0, Math.min(pool.size(), param.getSize() - result.size())));
        }

        return new RankResult(result);
    }

    private ScoreParam convertToScoreParam(RankParam param) {
        ScoreParam scoreParam = new ScoreParam();
        scoreParam.setAccountName(param.getAccountName());
        scoreParam.setContents(param.getContents());
        return scoreParam;
    }

    private List<Content> deduplication(List<Content> contents) {
        List<String> titles = new ArrayList<>();
        List<Content> result = new ArrayList<>();
        // 遍历所有列表
        for (Content c : contents) {
            if (similarity(c.getTitle(), titles)) {
                continue;
            } else {
                result.add(c);
                titles.add(c.getTitle());
            }
        }

        return result;
    }

    private boolean similarity(String title, List<String> titles) {
        return TitleSimilarCheckUtil.isDuplicateContent(title, titles);
    }

}
